import java.util.Vector;
import java.util.Random;
import java.util.Enumeration;

/**
 * This class is responsible for building data structures from records
 *  of the slog file. Each record read by SlogReader is passed here and
 * RecordHandler sifts out the relevent data from it.
 * 
 * Once the slog file is read, 4 data structures exist.
 * 
 * 1. (stateDefs) This is a vector of all the state definitions in the 
 *    order they were read. 
 * 2. Each of the state definition objects mentioned above contains a vector
 *    'stateVector' that is a collection of states of the same type
 *    in the ascending order of their end timestamps.
 * 3. (data) This is vector or all states of all types in ascending order 
 *    of their end timestamps.
 * 4. (a) This is an object that contains a vector 'arrows' which is a 
 *    list of ArrowInfo objects representing messages.
 */

public class RecordHandler {
    SLOG_InputStream slog = null;
    SLOG_Frame slog_frame = null;

    //  Omar's original RecordHandler data structure.
    Vector stateDefs;
    Vector data;
    Vector mtns;
    CLOG_ARROW a;

    /**
     * Constructor
     */
    public RecordHandler( final SLOG_InputStream in_slog, 
                          final SLOG_Frame       in_frame,
                                int              connect_idx,
                                int              view_idx )
    {
        slog       = in_slog;
        slog_frame = in_frame;
        Init();
        CreateStateDefs();
        CreateStates( connect_idx, view_idx );
    }

    private void Init()
    {
        stateDefs = new Vector ();
        data = new Vector ();
        mtns = new Vector ();
        a = new CLOG_ARROW ();
    }
  
    /**
     *  Scan SLOG_InputStream's profile to create the StateDefs[]
     */
    private void CreateStateDefs()
    {
        SLOG_ProfileEntry entry;

        Enumeration profile = slog.profile.entries.elements();
        while ( profile.hasMoreElements() ) {
            entry = ( SLOG_ProfileEntry ) profile.nextElement();
            CLOG_STATE statedef = new CLOG_STATE( entry );
            stateDefs.addElement( statedef );
        }
    }

    private void CreateStates( int connect_idx, int view_idx )
    {
        SLOG_Irec   irec;
        CLOG_STATE  statedef;
        Enumeration irecs;

        //   For Incoming Sector
        irecs = slog_frame.inc.irecs.elements();
        while ( irecs.hasMoreElements() ) {
            irec = ( SLOG_Irec ) irecs.nextElement();
            StateInfo state = new StateInfo( irec.fix,
                                             slog.threadinfos,
                                             view_idx );

            statedef = GetLinkToStateDefs( irec.fix.intvltype );
            state.SetStateDefLink( statedef );

            statedef.stateVector.addElement( state );    // For histogram
            data.addElement( state );
        }

        //   For Passing Through Sector
        irecs = slog_frame.pas.irecs.elements();
        while ( irecs.hasMoreElements() ) {
            irec = ( SLOG_Irec ) irecs.nextElement();
            StateInfo state = new StateInfo( irec.fix,
                                             slog.threadinfos,
                                             view_idx );

            statedef = GetLinkToStateDefs( irec.fix.intvltype );
            state.SetStateDefLink( statedef );

            statedef.stateVector.addElement( state );    // For histogram
            data.addElement( state );
        }

        //    For Current Sector
        irecs = slog_frame.out.irecs.elements();
        while ( irecs.hasMoreElements() ) {
            irec = ( SLOG_Irec ) irecs.nextElement();
            StateInfo state = new StateInfo( irec.fix,
                                             slog.threadinfos,
                                             view_idx );

            statedef = GetLinkToStateDefs( irec.fix.intvltype );
            state.SetStateDefLink( statedef );

            statedef.stateVector.addElement( state );    // For histogram
            data.addElement( state );
        }

        //    For Outgoing Sector
        irecs = slog_frame.cur.irecs.elements();
        while ( irecs.hasMoreElements() ) {
            irec = ( SLOG_Irec ) irecs.nextElement();
            StateInfo state = new StateInfo( irec.fix,
                                             slog.threadinfos,
                                             view_idx );

            statedef = GetLinkToStateDefs( irec.fix.intvltype );
            state.SetStateDefLink( statedef );

            statedef.stateVector.addElement( state );    // For histogram
            data.addElement( state );
        }
    }

    /**
     * This function traverses through the vector containing state definitions
     * looking for the state definition whose starting or ending 
     * etype corresponds with the given etype. If no such state def. is found
     * a null is returned.
     */
    private CLOG_STATE GetLinkToStateDefs( int in_intvltype ) 
    {
        Enumeration enum = stateDefs.elements();
        while ( enum.hasMoreElements () ) {
            CLOG_STATE statedef = ( CLOG_STATE ) enum.nextElement();
            if ( in_intvltype == statedef.intvltype ) 
                return statedef;
        }
        System.err.println ( "Error: Unrecognized interval type = "
                           + in_intvltype );
        return null;
    }

    protected void finalize() throws Throwable
    { 
        stateDefs   = null;
        data        = null;
        mtns        = null;
        a           = null;
        super.finalize();
    }
}
